mod conn;
pub mod entities;
mod migrator;
mod query;

use sea_orm::{ConnectOptions, Database, DatabaseConnection, DeriveActiveEnum, EnumIter};
use sea_orm_migration::MigratorTrait;
use serde::{Deserialize, Serialize};

pub use entities::{prelude::*, *};
use migrator::Migrator;

type Result<T> = std::result::Result<T, sea_orm::DbErr>;

#[derive(EnumIter, DeriveActiveEnum, Serialize, Deserialize, PartialEq, Eq, Clone, Debug)]
#[sea_orm(rs_type = "String", db_type = "String(Some(6))")]
pub enum PackageDepend {
    #[sea_orm(string_value = "depend")]
    Depend,
    #[sea_orm(string_value = "make")]
    Make,
    #[sea_orm(string_value = "check")]
    Check,
    #[sea_orm(string_value = "opt")]
    Opt,
}

#[derive(Serialize)]
pub struct FullPackage {
    #[serde(flatten)]
    entry: package::Model,
    licenses: Vec<String>,
    groups: Vec<String>,
    replaces: Vec<String>,
    provides: Vec<String>,
    depends: Vec<(PackageDepend, String)>,
    files: Vec<String>,
}

#[derive(Clone, Debug)]
pub struct RieterDb {
    conn: DatabaseConnection,
    pub pkg: query::PackageQuery,
    pub repo: query::RepoQuery,
    pub distro: query::DistroQuery,
}

impl RieterDb {
    pub async fn connect<C: Into<ConnectOptions>>(opt: C) -> Result<Self> {
        let db = Database::connect(opt).await?;

        Migrator::up(&db, None).await?;

        Ok(Self {
            conn: db.clone(),
            pkg: query::PackageQuery::new(db.clone()),
            repo: query::RepoQuery::new(db.clone()),
            distro: query::DistroQuery::new(db.clone()),
        })
    }
}
