use axum::{
    extract::{Path, Query, State},
    routing::get,
    Json, Router,
};

use super::pagination::{self, PaginatedResponse};
use crate::db;

pub fn router() -> Router<crate::Global> {
    Router::new()
        .route("/", get(get_packages))
        .route("/:id", get(get_single_package))
}

async fn get_packages(
    State(global): State<crate::Global>,
    Query(pagination): Query<pagination::Query>,
) -> crate::Result<Json<PaginatedResponse<db::package::Model>>> {
    let (total_pages, pkgs) = global
        .db
        .pkg
        .page(
            pagination.per_page.unwrap_or(25),
            pagination.page.unwrap_or(1) - 1,
        )
        .await?;

    Ok(Json(pagination.res(total_pages, pkgs)))
}

async fn get_single_package(
    State(global): State<crate::Global>,
    Path(id): Path<i32>,
) -> crate::Result<Json<crate::db::FullPackage>> {
    let entry = global
        .db
        .pkg
        .full(id)
        .await?
        .ok_or(axum::http::StatusCode::NOT_FOUND)?;

    Ok(Json(entry))
}
